"""
These are wrapper interfaces the link outside apps and/or the main package
together.

- These tools are allowed to handle cross-talk between services.
- These tools are NOT allowed to do calculations on the own that invoce some
  logic that belongs within another service.
"""
